#!/bin/bash

HELP_INFO='''
# Before Usage environment variables:
# FIPS_MODE needs to be set to one of the following disabled/enabled/strict/strong
#     More info for each mode can be found here - https://confluence.eng.vmware.com/display/CAEng/FIPS+Implementation+Notes
# EXTERNAL_CERT_PATH can be set to dir where the pem files to be imported. Default /run/external-certs
#
# Usage:
# source /usr/local/bin/setup_security.sh
#
# After that the following environment variables will be available for use
# * TRUST_STORE_PASSWORD - *deprecated* - contains the generated truststore password, will not be
#   changed if passed from outside. Will be removed in later versions.
# * JAVA_OPTS - will append the options needed to enable BC FIPS.
# * BCFIPS_JARS - the full path to bc-fips.jar or jars separated with :, so this can be added to
#   the start of application classpath like
#        java -cp ${BCFIPS_JARS:+"$BCFIPS_JARS":}$org_class_path
#
# The following files are located in the directory pointed by LIB_SECURITY_PATH
# * cacerts.pass - contains the password for the truststore in a format accepted by the Java keytool
'''

function print_help() {
  cat <<EOF
$HELP_INFO
EOF
}

JAVA_FIPS_SECURITY_OPTS=""

JAVA_SECURITY_PATH="$JAVA_HOME/conf/security"

DEFAULT_JAVA_SECURITY_FILE="$JAVA_SECURITY_PATH/java.security"

FIPS_MODE_DISABLED="disabled"
FIPS_MODE_ENABLED="enabled"
FIPS_MODE_STRICT="strict"
FIPS_MODE_STRONG="strong"

# set default BCFIPS_PROVIDER_PARAMS if not externally provided
if test -z "${BCFIPS_PROVIDER_PARAMS:-}"
then
  BCFIPS_PROVIDER_PARAMS='C:DEFRND[HMACSHA512(256)];HYBRID;ENABLE{ALL};'
fi


FIPS_SECURITY_PROVIDERS='''security.provider.1=org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider '''"$BCFIPS_PROVIDER_PARAMS"'''
# BCFIPS 1.0.2.1 cannot load without the SUN provider on second place
security.provider.2=SUN
security.provider.3=org.bouncycastle.jsse.provider.BouncyCastleJsseProvider fips:BCFIPS
security.provider.4=SunJSSE BCFIPS'''


if [ $# -eq 1 ] && [ "${1:-}" = "-h"  -o "${1:-}" = "--help" ]
then
  print_help
  exit 0
fi

# Defaults
EXTERNAL_CERT_PATH=${EXTERNAL_CERT_PATH:-/run/external-certs}
echo "EXTERNAL_CERT_PATH=$EXTERNAL_CERT_PATH"

#-------------------------------  OPTIONS --------------------------------------
if [ -z "${FIPS_MODE:-}" ]
then
  export FIPS_MODE="$FIPS_MODE_DISABLED"
  echo "WARNING: FIPS_MODE environment variable is not set! The default value is set to $FIPS_MODE"
else
  export FIPS_MODE=${FIPS_MODE,,}
fi
echo "FIPS_MODE=$FIPS_MODE"

if ! [[ "$FIPS_MODE" =~ ^($FIPS_MODE_DISABLED|$FIPS_MODE_ENABLED|$FIPS_MODE_STRICT|$FIPS_MODE_STRONG)$ ]]
then
  echo "ERROR: FIPS_MODE environment variable ($FIPS_MODE) is not set correctly! Use $FIPS_MODE_DISABLED/$FIPS_MODE_ENABLED/$FIPS_MODE_STRICT/$FIPS_MODE_STRONG"
  exit 2
fi

#--------------------------------- CHANGE PASSWORD -----------------------------
LIB_PATH="$JAVA_HOME/jre/lib"
test -e "$LIB_PATH" || LIB_PATH="$JAVA_HOME/lib"
test -e "$LIB_PATH/security-template/cacerts" || { echo "Cannot find cacerts template file in $LIB_PATH/security-template" ; exit 1 ; }

LIB_SECURITY_PATH="$LIB_PATH/security"
if test -e "$LIB_SECURITY_PATH/cacerts"
then
  echo "$LIB_SECURITY_PATH/cacerts already exists - will not overwrite from security-template"
else
  cp -Rp $LIB_PATH/security-template/* $LIB_SECURITY_PATH || { echo "Cannot find cacerts file in $JAVA_HOME" ; exit 1 ; }
fi
echo "Found cacerts in $LIB_SECURITY_PATH"
ls -al "$LIB_SECURITY_PATH" || ls "$LIB_SECURITY_PATH" || echo "Cannot list $LIB_SECURITY_PATH"

# Set randfile location to non-root user permissible folder
export RANDFILE="$LIB_SECURITY_PATH/.rnd"
CACERTS_PATH="$LIB_SECURITY_PATH/cacerts"
STOREPASS_FILE="$LIB_SECURITY_PATH/cacerts.pass"

if [ -f "$STOREPASS_FILE" ]
then
  echo "cacerts.pass was already initialized"
else
  # create files as read/write only to owner
  (
    umask 077
    openssl rand -base64 40 > "$STOREPASS_FILE"
  )

  keytool -storepasswd -keystore "$CACERTS_PATH" -storepass changeit -new:file "$STOREPASS_FILE"
fi

read TRUST_STORE_PASSWORD < "$STOREPASS_FILE"
export TRUST_STORE_PASSWORD

#-------------------------------IMPORT CERTIFICATES-----------------------------
echo "Using external cert path - $EXTERNAL_CERT_PATH"

for cert_file in "$EXTERNAL_CERT_PATH"/*.pem
do
  if test -f "$cert_file"
  then
    echo "Parsing $cert_file"

    # number of certs in the PEM file
    cert_count=$( grep -c -- '--END CERTIFICATE--' "$cert_file" )
    echo "Found $cert_count certificates in $cert_file"

    # For every cert in the PEM file, extract it and import into the JKS keystore
    # awk command: step 1, if line is in the desired cert, print the line
    #              step 2, increment counter when last line of cert is found
    for cert_index in $( seq 0 $((cert_count - 1)) )
    do
        cert_alias="ext-cert-$( basename "$cert_file" .pem )-$cert_index"
        echo "Importing certificate $cert_index alias '$cert_alias' from $cert_file"

        # delete previous cert with same alias in case restart
        keytool -delete -keystore "$CACERTS_PATH" \
                    -storepass:file "$STOREPASS_FILE" \
                    -noprompt \
                    -alias "$cert_alias" > /dev/null || true

        awk "n==$cert_index { print }; /--END CERTIFICATE--/ { n++ }" "$cert_file" \
        | keytool -import -trustcacerts -keystore "$CACERTS_PATH" \
            -storepass:file "$STOREPASS_FILE" \
            -noprompt \
            -alias "$cert_alias"
    done
  fi
done
unset cert_file cert_count cert_index cert_alias

#-------------------------------SETUP FIPS--------------------------------------
# Define JAVA_OPTS if it is not yet defined
JAVA_OPTS=${JAVA_OPTS:-}

if [[ "$FIPS_MODE" =~ ^($FIPS_MODE_ENABLED|$FIPS_MODE_STRICT|$FIPS_MODE_STRONG)$ ]]
then

  OVERRIDE_JAVA_SECURITY_FILE="$LIB_SECURITY_PATH/vmware-override-java.security"
  TMP_FILE="$LIB_SECURITY_PATH/tmp_file"
  TMP_FILE_BAK="$LIB_SECURITY_PATH/tmp_file.bak"

  echo "Setup $OVERRIDE_JAVA_SECURITY_FILE with bouncycastle security providers..."
  # Create vmware-override-java.security file based on default java.security
  cp $DEFAULT_JAVA_SECURITY_FILE $OVERRIDE_JAVA_SECURITY_FILE
  # Remove all lines containing security.provider.<any number>
  sed -i.bak '/security.provider.[0-9]/d' $OVERRIDE_JAVA_SECURITY_FILE
  # Add FIPS list of providers with their preference order
  awk -v security_providers="$FIPS_SECURITY_PROVIDERS" '{print} /# List of providers and their preference orders.*/{print security_providers}' $OVERRIDE_JAVA_SECURITY_FILE > $TMP_FILE

  # set default keystore type to bcfks
  echo "Set keystore.type to BCFKS"
  grep -q '^\s*keystore\.type\s*=' $TMP_FILE || echo 'keystore.type=bcfks' >> $TMP_FILE
  sed 's/\s*keystore\.type\s*=.*$/keystore.type=bcfks/' $TMP_FILE > $TMP_FILE_BAK
  cp $TMP_FILE_BAK $TMP_FILE

  echo "Set securerandom.strongAlgorithms to Java default one"
  grep -q '^\s*securerandom\.strongAlgorithms\s*=' $TMP_FILE || echo 'securerandom.strongAlgorithms=NativePRNGBlocking:SUN' >> $TMP_FILE
  sed 's/^\(securerandom\.strongAlgorithms\s*=\s*\).*$/\1NativePRNGBlocking:SUN/' $TMP_FILE > $TMP_FILE_BAK
  cp $TMP_FILE_BAK $TMP_FILE
  cat $TMP_FILE > $OVERRIDE_JAVA_SECURITY_FILE

  if [[ "$FIPS_MODE" =~ ^($FIPS_MODE_STRICT|$FIPS_MODE_STRONG)$ ]]
  then
    JAVA_FIPS_SECURITY_OPTS+=" -Dorg.bouncycastle.fips.approved_only=true"
  fi
  JAVA_FIPS_SECURITY_OPTS+=" -Djava.security.properties==$OVERRIDE_JAVA_SECURITY_FILE"
  BCFIPS_JARS=$( echo "$BCFIPS_DIR/"bc*-fips-*.jar | tr ' ' ':' )
  echo "Defined BCFIPS_JARS='$BCFIPS_JARS' to be added java class path."

#-------------------------------CREATE BC STORE-----------------------------------------

  BCFKS_TRUSTSTORE="$LIB_SECURITY_PATH/truststore.bcfks"
  BCFKS_TRUSTSTOREPASS_FILE="$STOREPASS_FILE"

  keytool -importkeystore -srckeystore $CACERTS_PATH -destkeystore $BCFKS_TRUSTSTORE \
    -noprompt \
    -srcstoretype JKS -deststoretype BCFKS \
    -srcstorepass:file $STOREPASS_FILE -deststorepass:file $BCFKS_TRUSTSTOREPASS_FILE \
    -provider org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider \
    -providerpath "$BCFIPS_JARS"

  JAVA_FIPS_SECURITY_OPTS+=" -Djavax.net.ssl.keyStoreType=BCFKS"
  JAVA_FIPS_SECURITY_OPTS+=" -Djavax.net.ssl.trustStoreType=BCFKS"
  JAVA_FIPS_SECURITY_OPTS+=" -Djavax.net.ssl.trustStore=$BCFKS_TRUSTSTORE"

  JAVA_TLS_CLIENT_PROTOCOLS="${JAVA_TLS_CLIENT_PROTOCOLS:-${JAVA_TLS_CLIENT_PROTOCOL:-}}"
  if [ -z "${JAVA_TLS_CLIENT_PROTOCOLS:-}" ]
  then
    # set to default protocols
    JAVA_TLS_CLIENT_PROTOCOLS="TLSv1.3,TLSv1.2"
  fi
fi # FIPS_MODE

JAVA_TLS_CLIENT_PROTOCOLS="${JAVA_TLS_CLIENT_PROTOCOLS:-${JAVA_TLS_CLIENT_PROTOCOL:-}}"
if [ -n "${JAVA_TLS_CLIENT_PROTOCOLS:-}" ]
then
  JAVA_FIPS_SECURITY_OPTS+=" -Djdk.tls.client.protocols=$JAVA_TLS_CLIENT_PROTOCOLS"
fi

if [ -n "${JAVA_FIPS_SECURITY_OPTS:-}" ]
then
  JAVA_OPTS+=" $JAVA_FIPS_SECURITY_OPTS"
  echo "'$JAVA_FIPS_SECURITY_OPTS' added to JAVA_OPTS environment..."
fi
